from setuptools import setup, find_packages


setup(
    name='avn_helpers',
    version='0.0.2',
    license='MIT',
    author="Muaz Twaty",
    author_email='email@example.com',
    packages=find_packages(),
    keywords='custom helpers',
    install_requires=[
        'setuptools~=57.0.0',
        'aiohttp==3.6.2',
        'apache-beam==2.24.0',
        'async-timeout==3.0.1',
        'attrs==20.2.0',
        'avro-python3==1.9.2.1',
        'cachetools==3.1.1',
        'certifi==2020.6.20',
        'cffi==1.14.3',
        'chardet==3.0.4',
        'crcmod==1.7',
        'dateparser==0.7.6',
        'dill==0.3.1.1',
        'docopt==0.6.2',
        'fastavro==0.23.6',
        'fasteners==0.15',
        'future==0.18.2',
        'google-api-core==1.22.3',
        'google-apitools==0.5.31',
        'google-auth==1.22.0',
        'google-cloud-bigquery==1.28.0',
        'google-cloud-bigquery-datatransfer==3.2.0',
        'google-cloud-bigtable==1.5.0',
        'google-cloud-core==1.4.2',
        'google-cloud-datastore==1.15.1',
        'google-cloud-dlp==1.0.0',
        'google-cloud-language==1.3.0',
        'google-cloud-pubsub==1.7.0',
        'google-cloud-spanner==1.19.0',
        'google-cloud-videointelligence==1.15.0',
        'google-cloud-vision==1.0.0',
        'google-crc32c==1.0.0',
        'google-resumable-media==1.0.0',
        'googleapis-common-protos==1.52.0',
        'grpc-google-iam-v1==0.12.3',
        'grpcio==1.32.0',
        'grpcio-gcp==0.2.2',
        'hdfs==2.5.8',
        'httplib2==0.17.4',
        'idna==2.10',
        'idna-ssl==1.1.0',
        'mock==2.0.0',
        'monotonic==1.5',
        'multidict==4.7.6',
        'numpy==1.19.2',
        'oauth2client==3.0.0',
        'pbr==5.5.0',
        'protobuf==3.13.0',
        'pyarrow==0.17.1',
        'pyasn1==0.4.8',
        'pyasn1-modules==0.2.8',
        'pycparser==2.20',
        'pydot==1.4.1',
        'pymongo==3.11.0',
        'PyMySQL==0.10.1',
        'pyparsing==2.4.7',
        'python-dateutil==2.8.1',
        'pytz==2020.1',
        'redis==3.5.3',
        'regex==2020.9.27',
        'requests==2.24.0',
        'rsa==4.6',
        'six==1.15.0',
        'typing-extensions==3.7.4.3',
        'tzlocal==2.1',
        'urllib3==1.25.10',
        'yarl==1.6.0',
        'SQLAlchemy',
        'alembic',
        'sqlalchemy_utils',
        'mysqlclient',
        'python-dotenv'
    ],
)
